#!/bin/bash

ME=`basename $0`
WHERE=`dirname $0`
STATUSURL="/jc"
CLICKURL="/cc"
CURLOPTS="-s -m 10"

write-debug() {
  if [ -n "$OG_DEBUG" ] ; then
    echo "$1" >> debug.log
  fi
}

Usage() {
  echo "$ME - opengarage utility"
  echo 
  echo "Usage:  $ME <site> <device_key>"
  echo
  echo "  <site> is the opengarage device IP address or name including the protocol (http://)"
  echo "  <device_key> is the key set in the opengarage device.  Defaults to 'opendoor' but you should change it."
  echo
}

keycheck() {
  if [ -z "$DKEY" ] ; then
    echo "No device key specified!" >&2
    exit 2
  fi
}

doorstatus() {
  doorstatus_return=`gstatus $1 | grep door`
  doorstatus_r=$?
  if [ "$doorstatus_r" = 0 ] ; then
    echo $doorstatus_return | awk -F: '{print $2;}'
  fi
  return $doorstatus_r
}

gstatus() {
  gstatus_site=$1
  if [ -z "$1" ] ; then
    echo "No site specified to doorstatus !" >&2
    return 4
  fi
  R=`curl $CURLOPTS "${gstatus_site}${STATUSURL}" | tr -d '{' | tr -d '}' | tr ',' '\n'` 
  if [ "$?" != "0" ] ; then
    echo "Error calling curl to $gstatus_site!" >&2
    exit 3
  fi
  if [ -z "$R" ] ; then
    echo "No device found at $gstatus_site" >&2
    return 1
  fi
  echo "$R"
  return 0
}

door() {
  device=$1
  action=$2
  if [ "$action" = "0" ] ; then
    opposite=1
  else 
    opposite=0
  fi
  DOORSTATUS=`doorstatus $device`
  if [ "$?" != 0 ] ; then
    if [ -z "$OG_SCAN" ] ; then
      echo "No device found at $device" >&2
      return 1
    elif [ -n "$OG_SCAN" ] ; then
      echo "No device found at $device.  Scanning . . ."
      NEWIP="`scan`"
      DOORSTATUS=`doorstatus $NEWIP`
      if [ "$DOORSTATUS" = "$opposite" ] ; then
        if [ -z "$OG_DEBUG" ] ; then
	  click $device
        fi
      else
        echo "Door is already in proper state."
      fi
    fi
  elif [ "$DOORSTATUS" = "$action" ] ; then
    echo "Door is already in proper state"
  elif [ "$DOORSTATUS" = "$opposite" ] ; then
    echo "Toggle Door"
    if [ -z "$OG_DEBUG" ] ; then
      click $device
    fi
  else
    echo "Error!.  Invalid state: $DOORSTATUS" >&2
    exit 6
  fi
}

opendoor() {
  door $1 1
}

closedoor() {
  door $1 0
}

click() {
  if [ -z "$OG_DEBUG" ] ; then
    curl -s "${1}${CLICKURL}?dkey=$DKEY&click=1" >/dev/null
  fi
}

scan(){
  if [ -z "$BASEIP" ] ; then
    echo "Error. NO BASEIP!  check opengarage.conf?"
    exit 1
  fi
  write-debug "Scanning network for opengarage devices . . ."
  if [ -z "$STARTIP" ] ; then
    STARTIP=0
  fi
  if [ -z "$ENDIP" ] ; then
    ENDIP=254
  fi
  CHECKIP=$STARTIP
  while [ "$CHECKIP" -lt  "$ENDIP" ] ; do
    MYIP="${BASEIP}${CHECKIP}"
    write-debug "Scanning $MYIP . . ."
    scan_status=`doorstatus ${MYIP} 2>/dev/null`
    if [ "$?" = 0 ] ; then
      echo "$scan_status $MYIP"
    fi
    CHECKIP=$(($CHECKIP + 1))
  done
}

if [ -f "/etc/opengarage-tools.conf" ] ; then
  .  /etc/opengarage-tools.conf
fi
SITE=$1
DKEY=$2

if [ "$1" = "-h" ] || [ "$1" = "--help" ] ; then
  Usage
  exit 0
fi

if [ "$ME" = "scan.sh" ] ; then
  scan
  exit
fi
if [ -z "$SITE" ] ; then
  echo "No site specified:  ex.  http://192.168.0.1" >&2
  exit 1
fi

if [ -n "$OG_DEBUG" ] ; then
  echo "DEBUG ON"
fi

if [ "$ME" = "status.sh" ] ; then
  gstatus $SITE
elif [ "$ME" = "closedoor.sh" ] ; then
  keycheck
  door $SITE 0
elif [ "$ME" = "opendoor.sh" ] ; then
  keycheck
  door $SITE 1
fi
